package board;

import java.awt.event.*;
import java.awt.*;
import java.util.*;
import javax.swing.Timer; 

class MainPanel extends Panel implements ButtonsInterface{

   MainPanel(){
      MouseDispatcher mouseDispatcher = new MouseDispatcher();
      addMouseListener(mouseDispatcher);
      addMouseMotionListener(mouseDispatcher);
		
		
      data  = Loader.load("data.csv");

      new Timer(1000, new ActionListener(){
            public void actionPerformed(ActionEvent ev){
               repaint();
            }
         }).start();

   }
   ArrayList<Tuple> data;
	
	
	

   public void initializeButtons(Buttons buttons){
      buttons.addButton("start");
      buttons.addButton("stop");
   }
   public void buttonPressed(String label){
      if (label.equals("start")){
         start_time = System.currentTimeMillis();
         timer_on = true;
      }
      if (label.equals("stop")){
         timer_on = false;
      }
   }	
	
	
   long start_time = System.currentTimeMillis();
   long current_time = System.currentTimeMillis();
   boolean timer_on = false;
   void paint_time(Graphics g){
      if (timer_on)
         current_time = System.currentTimeMillis();
		
      int time = (int) (current_time - start_time)/1000;
      g.setColor(Color.red);
      g.setFont(font);
      g.drawString(""+time, 5,25);
   }
   Font font = new Font("Helvetica", Font.BOLD, 20);
	
	
	
	
	
   Image buffered_image = null;
   public void update(Graphics g) {
      paint(g);
   }
   public boolean repaint_all = true;
   public void repaint_all(){
      repaint_all = true;
      repaint();
   }
   public void paint(Graphics g) {

      //double buffering
      if (buffered_image == null)
         buffered_image = createImage(getSize().width, getSize().height);
      Graphics bg = buffered_image.getGraphics();

      // clear
      bg.setColor(Color.white);
      bg.fillRect(0,0, getSize().width, getSize().height);

      // paint code!
      paint_chart(bg);
      paint_time(bg);
      paint_label(bg);
			
      //double buffering
      bg.dispose();
      
      g.drawImage(buffered_image,0,0,this);

   }
	
   void paint_chart(Graphics bg){
      bg.setColor(Color.black);

      for(int i=0; i<data.size(); i++){
         int x = convert_x(i);
         Tuple tuple = data.get(i);
         if (tuple.value == 0)
            continue;
         int y = convert_y(tuple.value);
         bg.drawLine(x, y, x, y+1);
      }
   }
	
	
   int convert_y(double value){
      return (int)(600 - value * 3);
   }
   int convert_x(int index){
      return offset + index;
   }
   int convert_x_inverse(int x){
      return x-offset;
   }
	

   
   String label = "";
   int selected_index = -1;
   void paint_label(Graphics g){
      g.setColor(Color.red);
      g.setFont(font);
      g.drawString(""+label, 600,25);
		
      if (selected_index != -1){
         int x = convert_x(selected_index);
         Tuple tuple = data.get(selected_index);
         if (tuple.value != 0){
            int y = convert_y(tuple.value);
            g.drawLine(x, y-5, x, y-15);
         }
      }
		
   }
   void update_label(Point p){
      int index = convert_x_inverse(p.x);
      if (index >=0 && index < data.size()){
         label = data.get(index).date +"  "+data.get(index).value;
         selected_index = index;
      }
      else {
         label = "";
         selected_index = -1;
      }
      repaint();
   }


	
	
   int offset = 0;
   Point prev_p = null;
   void start_dragging(Point p){
      prev_p = p;
   }
   void dragging(Point p){
      offset += p.x - prev_p.x;
      prev_p = p;
      repaint();
   }
   void finish_dragging(Point p){
		
   }
	
	
	
	
	
	

	
	
	
	
   
	
   public int operation_status;
   public static final int NONE = 0;
   public static final int INSPECTING = 1;
   public static final int DRAGGING = 2;
   
   //   
   // central event dispatcher
   //
   public class MouseDispatcher 
      extends MouseAdapter implements MouseMotionListener{

      public void mouseMoved(MouseEvent e) {
      }
      public void mousePressed(MouseEvent e) {
         Point p = e.getPoint();
         boolean right_button = (e.getModifiers() & e.BUTTON3_MASK) != 0;
			
         if (right_button){
            operation_status = DRAGGING;
            start_dragging(p);
         }
         else {
            operation_status = INSPECTING;
            update_label(e.getPoint());
				
         }
      }
      public void mouseDragged(MouseEvent e) {
         Point p = e.getPoint();
			
         switch(operation_status){
            case DRAGGING:
               dragging(p);
               break;
            case INSPECTING:
               update_label(e.getPoint());
               break;
         }
			
      }
      public void mouseReleased(MouseEvent e) {
         Point p = e.getPoint();
			
         switch(operation_status){
            case DRAGGING:
               finish_dragging(p);
               break;
         }
      }
      public void mouseClicked(MouseEvent e){
			
      }
   }
	
	
	

	
   public static Frame frame;
   public static void main(String args[]) {
      frame = new Frame("Main");
      frame.addWindowListener(
         new WindowAdapter(){
            public void windowClosing(WindowEvent e) {System.exit(0);}}
         );
      frame.addKeyListener(
         new KeyAdapter(){
            public void keyPress(KeyEvent e) {
               if (e.getKeyCode() == e.VK_C)
                  System.exit(0);}}
         );

      frame.setLayout(new BorderLayout());
      Panel panel = new Panel();
      panel.setLayout(new BorderLayout());
      MainPanel mainPanel = new MainPanel();
      panel.add("Center", mainPanel);
      panel.add("South",new Buttons(mainPanel));
		
      frame.add("Center", panel);
      frame.setSize(800, 600);
      frame.setVisible(true);
      //frame.pack();
   }
}







